
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.039.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "PayPal SOAP API client with sandbox support",
  "dist_author" => [
    "Scott Wiersdorf <scott\@perlcode.org>",
    "Danny Hembree <danny\@dynamical.org>",
    "Bradley M. Kuhn <bkuhn\@ebb.org>"
  ],
  "dist_name" => "Business-PayPal-API",
  "dist_version" => "0.71_02",
  "license" => "perl",
  "module_name" => "Business::PayPal::API",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::Printer" => 0,
    "SOAP::Lite" => "0.67",
    "perl" => "5.008001",
    "strict" => 0,
    "warnings" => 0
  },
  "test_requires" => {
    "Cwd" => 0,
    "List::AllUtils" => 0,
    "Test::More" => 0,
    "Test::Most" => 0,
    "autodie" => 0
  }
);


my %fallback_build_requires = (
  "Cwd" => 0,
  "List::AllUtils" => 0,
  "Module::Build" => "0.28",
  "Test::More" => 0,
  "Test::Most" => 0,
  "autodie" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
